from typing import List
from uuid import UUID

from pydantic import UUID4

NULL_UUID_LITERAL = "00000000-0000-0000-0000-000000000000"
NULL_UUID_V4_LITERAL = "00000000-0000-4000-8000-000000000000"

NULL_UUID_LITERALS: List[str] = [NULL_UUID_LITERAL, NULL_UUID_V4_LITERAL]

NULL_UUID: UUID = UUID(NULL_UUID_LITERAL)
NULL_UUID_V4: UUID4 = UUID(NULL_UUID_V4_LITERAL)

NULL_UUIDS: List[UUID] = [NULL_UUID, NULL_UUID_V4]
